package io.skore.smooch_plugin

import android.app.Activity
import android.content.Intent
import android.util.Log
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.skore.smooch_plugin.util.LangPreference
import io.skore.smooch_plugin.util.SmoochUtil
import io.skore.smooch_plugin.view.ACTION_MINIMIZE_SMOOCH
import io.skore.smooch_plugin.view.ChatActivity
import io.smooch.core.Smooch
import io.smooch.core.User
import java.lang.ref.WeakReference
import java.util.*

class SmoochPlugin(private val activity: Activity, private val methodChannel: MethodChannel) : MethodCallHandler {
    private val tag = SmoochPlugin::class.java.simpleName

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            Log.d("SmoochPlugin", "Creating method channel...")

            val channel = MethodChannel(registrar.messenger(), "smooch_plugin")
            channel.setMethodCallHandler(SmoochPlugin(registrar.activity(), channel))
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "init" -> {
                val appId = call.argument<String>("appId")
                val userId = call.argument<String>("userId")
                val smoochToken = call.argument<String>("smoochToken")
                val userProp = call.argument<HashMap<String, Any>>("properties")
                if (appId.isNullOrBlank() || userId.isNullOrBlank() || smoochToken.isNullOrBlank()) {
                    result.error("ERROR", "Invalid arguments", null)
                    return
                }
                init(SmoochUtil.Auth(appId, userId, smoochToken, userProp), result)
            }
            "show" -> {
                val lang = call.argument<String>("lang")
                val shouldSendStart = call.argument<Boolean>("sendStart") ?: false
                if (lang.isNullOrBlank()) {
                    result.error("ERROR", "Invalid arguments", null)
                    return
                }
                LangPreference.setUserLang(activity, lang)
                ChatActivity.start(activity, shouldSendStart)
                result.success(0)
            }
            "logout" -> {
                Smooch.logout {
                    Log.d(tag, "Smooch logout status: ${it.status} error: ${it.error}")
                }
                result.success(0)
            }
            "minimize" -> {
                LocalBroadcastManager.getInstance(activity).sendBroadcast(Intent(ACTION_MINIMIZE_SMOOCH))
                result.success(0)
            }
            else -> result.notImplemented()
        }
    }

    private fun init(auth: SmoochUtil.Auth, result: Result) {
        SmoochUtil.login(activity.application, auth) { isLoggedIn ->
            if (isLoggedIn) {
                if (!auth.userProp.isNullOrEmpty()) {
                    User.getCurrentUser().addProperties(auth.userProp)
                }

                SmoochUtil.addDelegates(methodChannel)
//                SmoochUtil.setAuth(activity, auth)

                SmoochUtil.methodChannelRef = WeakReference(methodChannel)

                result.success(0)
            } else {
                result.success(-1)
            }
        }
    }
}
