package io.skore.smooch_plugin.notification

import android.util.Log
import com.google.firebase.messaging.FirebaseMessagingService
import com.google.firebase.messaging.RemoteMessage
import io.skore.smooch_plugin.view.ChatActivity
import org.json.JSONObject

class FirebaseMessaging : FirebaseMessagingService() {
    private val tag = FirebaseMessaging::class.java.simpleName
    
    override fun onMessageReceived(remoteMessage: RemoteMessage) {
        super.onMessageReceived(remoteMessage)

        with (remoteMessage) {
            Log.d(tag, "Firebase received msg(${messageType ?: "unknown"}) from: ${from ?: "unknown"}")
            for (param in data) {
                Log.d(tag, "${param.key}: ${param.value}")
            }

            if (ChatActivity.isChatbotResumed) {
                Log.d(tag, "Chatbot is resumed, skipping notification...")
                return
            }

            val msg = JSONObject(data["message"])
            val avatarUrl = msg.optString("avatarUrl")
            val name = msg.optString("name")
            val text = msg.optString("text")

            if (name.isNullOrBlank() || text.isNullOrBlank() || avatarUrl.isNullOrBlank()) {
                Log.e(tag, "Missing notification parameters.")
                return
            }

            NotificationTask(applicationContext, name, text).execute(avatarUrl)
        }
    }
}