package io.skore.smooch_plugin.view

import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.WindowManager
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import io.skore.smooch_plugin.util.LangPreference
import io.skore.smooch_plugin.util.SmoochUtil
import io.smooch.core.Message
import io.smooch.core.Smooch
import io.smooch.ui.ConversationActivity
import kotlinx.android.synthetic.main.smooch_activity_conversation.*

const val ACTION_MINIMIZE_SMOOCH = "ACTION_MINIMIZE_SMOOCH"
const val ACTION_UNKNOWN = "ACTION_UNKNOWN"
const val SEND_START = "SEND_START"

class ChatActivity : ConversationActivity() {
    private val tag = ChatActivity::class.java.simpleName

    companion object {
        var isChatbotResumed = false

        @JvmStatic
        fun start(activity: Activity, shouldSendStart: Boolean) {
            val intent = Intent(activity, ChatActivity::class.java).apply {
                putExtra(SEND_START, shouldSendStart)
            }
            activity.startActivity(intent)
        }
    }

    private val broadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            Log.d(tag, "Broadcast received: ${intent?.action ?: ACTION_UNKNOWN}")

            if (isChatbotResumed && intent?.action == ACTION_MINIMIZE_SMOOCH) {
                onBackPressed()
            }
        }
    }

    override fun attachBaseContext(baseContext: Context?) {
        super.attachBaseContext(if (baseContext != null) LangPreference.updateResources(baseContext, LangPreference.getLang(baseContext)) else null)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        window.setFlags(WindowManager.LayoutParams.FLAG_SECURE, WindowManager.LayoutParams.FLAG_SECURE)

        LocalBroadcastManager.getInstance(this).registerReceiver(broadcastReceiver, IntentFilter(ACTION_MINIMIZE_SMOOCH))

        intent?.apply {
            val shouldSendStart = getBooleanExtra(SEND_START, false)
            if (shouldSendStart) {
                removeExtra(SEND_START)
                Smooch.getConversation()?.sendMessage(Message("iniciar"))
            }
        }

        logSmoochStatus()

        supportActionBar?.setDisplayHomeAsUpEnabled(false)

        back.setOnClickListener {
            onBackPressed()
        }

        help.setOnClickListener {
            SmoochUtil.methodChannelRef?.get()?.invokeMethod("onHelpClick", null, null)
        }
    }

    override fun onDestroy() {
        super.onDestroy()

        LocalBroadcastManager.getInstance(this).unregisterReceiver(broadcastReceiver)
    }

    override fun onResume() {
        super.onResume()
        Log.d(tag, "isChatbotResumed = true")
        isChatbotResumed = true
    }

    override fun onPause() {
        super.onPause()
        Log.d(tag, "isChatbotResumed = false")
        isChatbotResumed = false
    }

    private fun setHelpStatus(isLoading: Boolean) {
        with(layout_help) {
            if (isLoading) {
                isClickable = false
                visibility = View.GONE
                progress_help.visibility = View.VISIBLE
            } else {
                isClickable = true
                visibility = View.VISIBLE
                progress_help.visibility = View.GONE
            }
        }
    }

    private fun logSmoochStatus() {
        val connectionStatus = Smooch.getSmoochConnectionStatus()?.name ?: "unknown"
        val initStatus = Smooch.getInitializationStatus()?.name ?: "unknown"
        val loginStatus = Smooch.getLastLoginResult()?.name ?: "unknown"
        Log.d(tag, "Smooch status => Connection: $connectionStatus Init: $initStatus Login: $loginStatus")
    }
}