import Flutter
import UIKit
import Smooch


public class SwiftSmoochPlugin: NSObject, FlutterPlugin {

    var channel : FlutterMethodChannel?

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "smooch_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftSmoochPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if call.method == "init" {
            initialize(call.arguments, result: result)
            return
        }

        if call.method == "show" {
            show(call.arguments, result: result)
            return
        }

        if call.method == "logout" {
            logout(result: result)
            return
        }

        if call.method == "minimize" {
            minimize(call.arguments, result: result)
            return
        }

        result(FlutterMethodNotImplemented)
    }

}

// MARK: - FlutterMethodCalls
extension SwiftSmoochPlugin {

    private func initialize(_ arguments: Any?, result: @escaping FlutterResult) {

        guard let arguments = arguments as? [String: Any] else {
            result(-1)
            return
        }

        #if DEBUG
        print(arguments)
        #endif

        SmoochHandler.shared().setMenuItemsAllowed(arguments["menuItemsAllowed"])
        SmoochHandler.shared().setChatColor(arguments["chatColor"])

        if let appId = arguments["appId"] as? String,
            let userId = arguments["userId"] as? String,
            let smoochToken = arguments["smoochToken"] as? String {

            SmoochHandler.shared().app_id = appId
            SmoochHandler.shared().smooch_token = smoochToken

            let settings = SKTSettings(appId: appId)
            if let chatColor = SmoochHandler.shared().chatColor {
                settings.conversationAccentColor = chatColor
            }
            settings.conversationStatusBarStyle = .lightContent;
            settings.allowedMenuItems = SmoochHandler.shared().menuItensAllowed
            settings.enableAppDelegateSwizzling = false
            settings.enableUserNotificationCenterDelegateOverride = false

            Smooch.destroy()

            Smooch.initWith(settings) { error, dictionary in
                if let error = error {
                    print(error.localizedDescription)
                    result(-1)
                    return
                }
                let user = "\(userId)"
                Smooch.login(user, jwt: smoochToken, completionHandler: { error, info in

                    if let loginError = error {
                        print(loginError.localizedDescription)
                    }

                    if let loginInfo = info {
                        print(loginInfo)
                    }

                    SmoochHandler.shared().smoochInited = true
                    SmoochHandler.shared().conversation = Smooch.conversation()
                    result(1)
                })
            }
        }
    }

    private func show(_ arguments: Any?, result: @escaping FlutterResult) {
        if let arguments = arguments as? [String: Any],
            let properties = arguments["properties"] as? [String: Any] {
            SmoochHandler.shared().properties = properties
        }
        SmoochHandler.shared().show(channel) { (res) in
            result(res)
        }
    }

    private func logout(result: @escaping FlutterResult) {
        Smooch.logout(completionHandler: nil)
    }

    private func minimize(_ arguments: Any?, result: @escaping FlutterResult) {

        if let arguments = arguments {
            print("FlutterMethod \(#function) arguments: \(arguments)")
        }

        SmoochHandler.shared().closeConversarionViewController()
        result(nil)
    }

}
